//
//  ViewContorller.swift
//  adamo-sample
//
//  Created by Trần Tý on 8/12/19.
//  Copyright © 2019 Trần Tý. All rights reserved.
//

import TwilioVideo
import CallKit

class CallView: UIView {
    var accessToken: String = ""
    // Video SDK components
    var room: TVIRoom?
    /**
     * We will create an audio device and manage it's lifecycle in response to CallKit events.
     */
    var audioDevice: TVIDefaultAudioDevice = TVIDefaultAudioDevice()
    var camera: TVICameraSource?
    var localVideoTrack: TVILocalVideoTrack?
    var localAudioTrack: TVILocalAudioTrack?
    
    // `TVIVideoView` created from a storyboard
    var previewView: CallPreview?
    
    // State
    @objc var shareVideo: Bool = true {
        didSet {
            toggleCamera()
        }
    }
    @objc var shareAudio: Bool = true {
        didSet {
            muteAudio()
        }
    }
    var canFlipCam: Bool = false
    
    var remoteParticipants: Array<TVIRemoteParticipant> = []
    
    // event
    
    @objc var onDidConnect: RCTBubblingEventBlock!
    @objc var onFailedToConnect: RCTDirectEventBlock!
    @objc var onDisConnected: RCTDirectEventBlock!
    @objc var onReConnecting: RCTDirectEventBlock!
    @objc var onFailedReConnect: RCTDirectEventBlock!
    @objc var onDidReConnect: RCTDirectEventBlock!
    @objc var onParticipantConnected: RCTDirectEventBlock!
    @objc var onParticipantDisConnected: RCTDirectEventBlock!
    @objc var onFlipCam: RCTDirectEventBlock!
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
        fatalError()
    }
    override init(frame: CGRect) {
        super.init(frame: frame)
        TwilioVideo.audioDevice = self.audioDevice;
        prepareLocalMedia()
    }
    
    func performRoomConnect(uuid: UUID, roomName: String?) {
        // Configure access token either from server or manually.
        // If the default wasn't changed, try fetching from server.
        
        // Preparing the connect options with the access token that we fetched (or hardcoded).
        let connectOptions = TVIConnectOptions.init(token: accessToken) { (builder) in
            
            // Use the local media that we prepared earlier.
            builder.audioTracks = self.localAudioTrack != nil ? [self.localAudioTrack!] : [TVILocalAudioTrack]()
            builder.videoTracks = self.localVideoTrack != nil ? [self.localVideoTrack!] : [TVILocalVideoTrack]()
            
            // Use the preferred audio codec
            if let preferredAudioCodec = Settings.shared.audioCodec {
                builder.preferredAudioCodecs = [preferredAudioCodec]
            }
            
            // Use the preferred video codec
            if let preferredVideoCodec = Settings.shared.videoCodec {
                builder.preferredVideoCodecs = [preferredVideoCodec]
            }
            
            // Use the preferred encoding parameters
            if let encodingParameters = Settings.shared.getEncodingParameters() {
                builder.encodingParameters = encodingParameters
            }
            
            // The name of the Room where the Client will attempt to connect to. Please note that if you pass an empty
            // Room `name`, the Client will create one for you. You can get the name or sid from any connected Room.
            builder.roomName = roomName            
            // The CallKit UUID to assoicate with this Room.
            builder.uuid = uuid
        }
        
        // Connect to the Room using the options we provided.
        room = TwilioVideo.connect(with: connectOptions, delegate: self)
        
        logMessage(messageText: "Attempting to connect to room \(String(describing: roomName))")
    }
    
    func connectWithOptions(token: String, room: String){
        accessToken = token
        performRoomConnect(uuid: UUID.init(), roomName: room)
    }
    
    func disconnect() {
        if let room = room {
            logMessage(messageText: "Attempting to disconnect from room \(room.name)")
            room.disconnect()
        }
    }
    
    func toggleCamera(){
        self.localVideoTrack?.isEnabled = shareVideo
    }
    
    func muteAudio() {
        if let localAudioTrack = self.localAudioTrack {
            localAudioTrack.isEnabled = self.shareAudio
        }
    }
    
    // MARK: Private
    func startPreview() {
        if PlatformUtils.isSimulator {
            return
        }
        
        let frontCamera = TVICameraSource.captureDevice(for: .front)
        let backCamera = TVICameraSource.captureDevice(for: .back)
        let hasCamera: Bool = frontCamera != nil || backCamera != nil
        if hasCamera {
            // Preview our local camera track in the local video preview view.
            camera = TVICameraSource(delegate: self)
            localVideoTrack = TVILocalVideoTrack.init(source: camera!, enabled: true, name: "Camera")
            
            // Add renderer to video track for local preview
            self.previewView?.addRenderer(track: localVideoTrack!)
            
            canFlipCam = frontCamera != nil && backCamera != nil
            
            camera!.startCapture(with: frontCamera != nil ? frontCamera! : backCamera!) { (captureDevice, videoFormat, error) in
                if let error = error {
                    self.logMessage(messageText: "Capture failed with error.\ncode = \((error as NSError).code) error = \(error.localizedDescription)")
                } else {
                    self.previewView?.preview.shouldMirror = (captureDevice.position == .front)
                }
                
            }
        }
    }
    
    func flipCamera() {
        if !canFlipCam {
            return
        }
        var newDevice: AVCaptureDevice?
        
        if let camera = self.camera, let captureDevice = camera.device {
            let isFront = captureDevice.position == .front
            newDevice = TVICameraSource.captureDevice(for: isFront ? .back : .front)
            
            if let newDevice = newDevice {
                camera.select(newDevice) { (captureDevice, videoFormat, error) in
                    let nextIsFront = !isFront
                    if let _ = error {
//                        self.onFlipCam(["success": false])
                        return
                    }
//                    self.onFlipCam(["success": true])
                    self.previewView?.preview.shouldMirror = nextIsFront
                }
            }
        }
    }
    
    func prepareLocalMedia() {
        // We will share local audio and video when we connect to the Room.
        // Create an audio track.
        if (localAudioTrack == nil) {
            localAudioTrack = TVILocalAudioTrack.init()
            
            if (localAudioTrack == nil) {
                logMessage(messageText: "Failed to create audio track")
            }
        }
        
        // Create a video track which captures from the camera.
        if (localVideoTrack == nil) {
            self.startPreview()
        }
    }
    
    
    func logMessage(messageText: String) {
        NSLog(messageText)
    }
}

// MARK: TVIRoomDelegate
extension CallView : TVIRoomDelegate {
    func didConnect(to room: TVIRoom) {
        // At the moment, this example only supports rendering one Participant at a time.
        logMessage(messageText: "Connected to room \(room.name) as \(String(describing: room.localParticipant?.identity))")
        remoteParticipants = room.remoteParticipants
        onDidConnect(["participants": TwilioVideoRctconvert.convertRemotePaticipants(self.remoteParticipants)])
        
    }
    
    func room(_ room: TVIRoom, didDisconnectWithError error: Error?) {
        remoteParticipants = []
        onDisConnected([:])
       
        self.room = nil
    }
    
    func room(_ room: TVIRoom, didFailToConnectWithError error: Error) {
        onFailedToConnect([:])
        logMessage(messageText: "Failed to connect to room with error: \(error.localizedDescription)")
        self.room = nil
    }
    
    func room(_ room: TVIRoom, isReconnectingWithError error: Error) {
        onFailedReConnect([:])
        logMessage(messageText: "Reconnecting to room \(room.name), error = \(String(describing: error))")
    }
    
    func didReconnect(to room: TVIRoom) {
        onDidReConnect([:])
        logMessage(messageText: "Reconnected to room \(room.name)")
    }
    
    func room(_ room: TVIRoom, participantDidConnect participant: TVIRemoteParticipant) {
        remoteParticipants.append(participant)
        onParticipantConnected(["participants": TwilioVideoRctconvert.convertRemotePaticipants(self.remoteParticipants)])
        logMessage(messageText: "Participant \(participant.identity) connected with \(participant.remoteAudioTracks.count) audio and \(participant.remoteVideoTracks.count) video tracks")
        
    }
    
    func room(_ room: TVIRoom, participantDidDisconnect participant: TVIRemoteParticipant) {
        remoteParticipants.removeAll { (par) -> Bool in
            return par.identity == participant.identity
        }
        onParticipantDisConnected(["participants": TwilioVideoRctconvert.convertRemotePaticipants(self.remoteParticipants)])
        logMessage(messageText: "Room \(room.name), Participant \(participant.identity) disconnected")
    }
}


// MARK: TVICameraSourceDelegate
extension CallView : TVICameraSourceDelegate {
    func cameraSource(_ source: TVICameraSource, didFailWithError error: Error) {
        logMessage(messageText: "Camera source failed with error: \(error.localizedDescription)")
    }
}
extension CallView {
    func settupPreview(_ pView: CallPreview){
        if self.previewView != nil {
            return
        }
        self.previewView = pView
        if localVideoTrack != nil {
            previewView?.addRenderer(track: localVideoTrack!)
        }
    }
}
